<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; //Exit if assessed directly
}

/**
 * WHMCS Fetch Data Action
 *
 * Ajax call for get data form WHMCS api.
 * It will fetch products with name , id , price, description.
 * It will use for a new price table Elementor widget.
 *
 * @since 2.2.3
 */


add_action( 'wp_ajax_fetch_whmcs_data', 'wdes_fetch_whmcs_data' );
add_action( 'wp_ajax_nopriv_fetch_whmcs_data', 'wdes_fetch_whmcs_data' );

function wdes_fetch_whmcs_data () {

    //check if url , id , secret has been added

    $url = wdes_opts_get('whmcs_url', '');
    $id = wdes_opts_get('whmcs_identifier', '');
    $secret = wdes_opts_get('whmcs_secret', '');

    if( ! wdes_button_action_check_validation( $url, $id,  $secret) ) return false;

    $args = [
        'identifier' => $id,
        'secret' => $secret,
        'action' => 'GetProducts',
        'responsetype' => 'json',
    ];

    $response = wdes_button_action_whmcs_api_call($url, $args);

    if( ! $response ) return false;

    $save_status = wdes_button_action_save_data( $response );

    if( ! $save_status ) return false;

    wp_send_json_success( ['type' => 'success save', 'message' => wdes_button_action_prepare_message('Connecting with WHMCS and fetching data has been completed successfully.')] );

}

function wdes_button_action_check_validation( $url, $id, $secret ){

    if( empty($url) &&  empty($id) && empty($secret)) {

        wp_send_json_error( ['type' => 'error validation', 'message' => wdes_button_action_prepare_message('You must enter data on (URL, ID, Secret) fields and save them before connecting with your WHMCS site.')]);

        return false;

    }

    if( ! filter_var($url, FILTER_VALIDATE_URL) ){

        wp_send_json_error( ['type' => 'error validation', 'message' => wdes_button_action_prepare_message('Please enter valid url')]);

        return false;
    }

    if( strlen( $id ) < 20 && strlen( $secret ) < 20 ){

        wp_send_json_error( ['type' => 'error validation', 'message' => wdes_button_action_prepare_message('Please enter valid ID & Secret')]);

        return false;

    }

    return true;

}

function wdes_button_action_prepare_message($message){

    return esc_html__($message, 'phox');

}


function wdes_button_action_whmcs_api_call( $url, $args = [] ){

    $api_url = esc_url(  $url . '/includes/api.php'  );

    $data = http_build_query( $args );

    $request_args = [
        'timeout'     => 30,
        'body'        => $data,
    ];

    $request = wp_remote_post( $api_url, $request_args );

    if( is_wp_error($request) ){

        wp_send_json_error( ['type' => 'error connection', 'message' => $request->get_error_message()]);

        return false;
    }

    $response_code = wp_remote_retrieve_response_code( $request );

    $response_body = wp_remote_retrieve_body( $request );

    if( $response_code !== 200 ){

        $message = explode('Authentication', $response_body);

        if( isset( $message[1] ) ){

            $message  = 'Authentication Failed';

        }

        wp_send_json_error( ['type' => 'error connection', 'message' => $message]);

        return false;
    }

    return json_decode($response_body, true);

}

function wdes_button_action_save_data($data){

    $option_value = [];

    $uids = [];

    if( $data['result'] !== 'success' && $data['totalresults'] < 0 ) {

        wp_send_json_error( ['type' => 'error after fetch data', 'message' => wdes_button_action_prepare_message('There are no products')]);

        return false;
    }

    $option = get_option( wdes_button_action_option_name(), '' );

    if( ! empty( $option ) ){

        wp_send_json_error( ['type' => 'error found option', 'message' => wdes_button_action_prepare_message('You need to remove old data first before fetch a new data')]);

        return false;

    }

    foreach ( $data['products']['product'] as $product ){

        $uid = uniqid( 'wdes'.$product['pid'].'_' );

        unset($product['customfields']);
        unset($product['configoptions']);

        $uids['wdes'.$product['pid']] = $uid;

        $option_value ['products'][$uid] = $product;

    }

    add_option( wdes_button_action_option_name(), $option_value );

    $get_posts = wdes_button_sql_get_posts();

    if( $get_posts ){

        $update_sql_data = wdes_button_sql_change_setting($get_posts, $uids);

        if( ! empty( $update_sql_data ) ){

            wdes_button_sql_update_meta_value($update_sql_data);

        }

    }

    return true;

}

function wdes_button_sql_get_posts(){

    global $wpdb;

    $posts_elementor_settings = [];

    $search_word = '%whmcs_settings_select_product%';
    $condition_status = 'inherit';

    $sql  = $wpdb->prepare( "SELECT * FROM $wpdb->postmeta where meta_value like %s and post_id IN (Select id from $wpdb->posts where post_status != %s)", $search_word, $condition_status );
    $results  = $wpdb->get_results( $sql );

    return ! empty( $results ) ? $results : false;

}

function wdes_button_sql_change_setting( array $sql_data, array $uids){

    $after_update_sql_data = [];

    foreach ( $sql_data as $post_setting ){

        $meta_value = json_decode($post_setting->meta_value, true);

        $update_meta_value = $meta_value;

        foreach ( $meta_value as $s_key => $section ){

            foreach ( $section['elements'] as $c_key => $column ){

                foreach ( $column['elements'] as $e_key => $element ) {

                    if( $element['widgetType'] === 'wdes_whmcs_price_table' ){

                        foreach ( $element['settings'] as $se_key => $setting ){

                            if( ! is_bool( strpos($se_key, 'title_wdes') ) || ! is_bool( strpos($se_key, 'feature_items_wdes') ) || ! is_bool( strpos($se_key, 'button_link__wdes') ) ) {

                                unset($update_meta_value[$s_key]['elements'][$c_key]['elements'][$e_key]['settings'][$se_key]);

                            }

                            if( $se_key === 'whmcs_settings_select_product' ){

                                $product_id = preg_split("/_[a-z 0-9]+/", $setting);

                                $update_meta_value[$s_key]['elements'][$c_key]['elements'][$e_key]['settings'][$se_key] = $uids[$product_id[0]];

                            }

                        }

                    }

                }

            }

        }

        $after_update_sql_data [$post_setting->meta_id] = json_encode($update_meta_value);

    }

    return $after_update_sql_data;

}

function wdes_button_sql_update_meta_value(  array $update_sql_data ){

    global $wpdb;

    foreach ( $update_sql_data as $meta_id => $meta_value ){

        $wpdb->update( $wpdb->postmeta, ['meta_value' => $meta_value], ['meta_id' => $meta_id ] );

    }


}

function wdes_button_action_option_name(){

    return 'wdes_whmcs_fetch_data';

}

